@@ -1,5 +1,5 @@
 /* Compile a Java program.
-   Copyright (C) 2001-2003, 2006-2016 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2006-2022 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <alloca.h>
@@ -98,10 +98,28 @@ default_target_version (void)
     {
       /* Determine the version from the found JVM.  */
       java_version_cache = javaexec_version ();
-      if (java_version_cache == NULL
-          || !(java_version_cache[0] == '1' && java_version_cache[1] == '.'
-               && (java_version_cache[2] >= '1' && java_version_cache[2] <= '6')
-               && java_version_cache[3] == '\0'))
+      if (java_version_cache == NULL)
+        java_version_cache = "1.1";
+      else if ((java_version_cache[0] == '1'
+                && java_version_cache[1] == '.'
+                && java_version_cache[2] >= '1' && java_version_cache[2] <= '8'
+                && java_version_cache[3] == '\0')
+               || (java_version_cache[0] == '9'
+                   && java_version_cache[1] == '\0')
+               || (java_version_cache[0] == '1'
+                   && (java_version_cache[1] >= '0'
+                       && java_version_cache[1] <= '1')
+                   && java_version_cache[2] == '\0'))
+        /* It's one of the valid target version values.  */
+        ;
+      else if (java_version_cache[0] == '1'
+               && (java_version_cache[1] >= '2'
+                   && java_version_cache[1] <= '7')
+               && java_version_cache[2] == '\0')
+        /* Assume that these (not yet released) Java versions will behave
+           like the preceding ones.  */
+        java_version_cache = "11";
+      else
         java_version_cache = "1.1";
     }
   return java_version_cache;
@@ -110,14 +128,25 @@ default_target_version (void)
 /* ======================= Source version dependent ======================= */
 
 /* Convert a source version to an index.  */
-#define SOURCE_VERSION_BOUND 3 /* exclusive upper bound */
+#define SOURCE_VERSION_BOUND 8 /* exclusive upper bound */
 static unsigned int
 source_version_index (const char *source_version)
 {
-  if (source_version[0] == '1' && source_version[1] == '.'
-      && (source_version[2] >= '3' && source_version[2] <= '5')
-      && source_version[3] == '\0')
-    return source_version[2] - '3';
+  if (source_version[0] == '1' && source_version[1] == '.')
+    {
+      if ((source_version[2] >= '3' && source_version[2] <= '5')
+          && source_version[3] == '\0')
+        return source_version[2] - '3';
+      if ((source_version[2] >= '7' && source_version[2] <= '8')
+          && source_version[3] == '\0')
+        return source_version[2] - '4';
+    }
+  else if (source_version[0] == '9' && source_version[1] == '\0')
+    return 5;
+  else if (source_version[0] == '1'
+           && (source_version[1] >= '0' && source_version[1] <= '1')
+           && source_version[2] == '\0')
+    return source_version[1] - '0' + 6;
   error (EXIT_FAILURE, 0, _("invalid source_version argument to compile_java_class"));
   return 0;
 }
@@ -132,6 +161,16 @@ get_goodcode_snippet (const char *source_version)
     return "class conftest { static { assert(true); } }\n";
   if (strcmp (source_version, "1.5") == 0)
     return "class conftest<T> { T foo() { return null; } }\n";
+  if (strcmp (source_version, "1.7") == 0)
+    return "class conftest { void foo () { switch (\"A\") {} } }\n";
+  if (strcmp (source_version, "1.8") == 0)
+    return "class conftest { void foo () { Runnable r = () -> {}; } }\n";
+  if (strcmp (source_version, "9") == 0)
+    return "interface conftest { private void foo () {} }\n";
+  if (strcmp (source_version, "10") == 0)
+    return "class conftest { public void m() { var i = new Integer(0); } }\n";
+  if (strcmp (source_version, "11") == 0)
+    return "class conftest { Readable r = (var b) -> 0; }\n";
   error (EXIT_FAILURE, 0, _("invalid source_version argument to compile_java_class"));
   return NULL;
 }
@@ -147,6 +186,16 @@ get_failcode_snippet (const char *source_version)
   if (strcmp (source_version, "1.4") == 0)
     return "class conftestfail<T> { T foo() { return null; } }\n";
   if (strcmp (source_version, "1.5") == 0)
+    return "class conftestfail { void foo () { switch (\"A\") {} } }\n";
+  if (strcmp (source_version, "1.7") == 0)
+    return "class conftestfail { void foo () { Runnable r = () -> {}; } }\n";
+  if (strcmp (source_version, "1.8") == 0)
+    return "interface conftestfail { private void foo () {} }\n";
+  if (strcmp (source_version, "9") == 0)
+    return "class conftestfail { public void m() { var i = new Integer(0); } }\n";
+  if (strcmp (source_version, "10") == 0)
+    return "class conftestfail { Readable r = (var b) -> 0; }\n";
+  if (strcmp (source_version, "11") == 0)
     return NULL;
   error (EXIT_FAILURE, 0, _("invalid source_version argument to compile_java_class"));
   return NULL;
@@ -155,14 +204,20 @@ get_failcode_snippet (const char *source_version)
 /* ======================= Target version dependent ======================= */
 
 /* Convert a target version to an index.  */
-#define TARGET_VERSION_BOUND 6 /* exclusive upper bound */
+#define TARGET_VERSION_BOUND 11 /* exclusive upper bound */
 static unsigned int
 target_version_index (const char *target_version)
 {
   if (target_version[0] == '1' && target_version[1] == '.'
-      && (target_version[2] >= '1' && target_version[2] <= '6')
+      && (target_version[2] >= '1' && target_version[2] <= '8')
       && target_version[3] == '\0')
     return target_version[2] - '1';
+  else if (target_version[0] == '9' && target_version[1] == '\0')
+    return 8;
+  else if (target_version[0] == '1'
+           && (target_version[1] >= '0' && target_version[1] <= '1')
+           && target_version[2] == '\0')
+    return target_version[1] - '0' + 9;
   error (EXIT_FAILURE, 0, _("invalid target_version argument to compile_java_class"));
   return 0;
 }
@@ -184,10 +239,36 @@ corresponding_classfile_version (const char *target_version)
     return 49;
   if (strcmp (target_version, "1.6") == 0)
     return 50;
+  if (strcmp (target_version, "1.7") == 0)
+    return 51;
+  if (strcmp (target_version, "1.8") == 0)
+    return 52;
+  if (strcmp (target_version, "9") == 0)
+    return 53;
+  if (strcmp (target_version, "10") == 0)
+    return 54;
+  if (strcmp (target_version, "11") == 0)
+    return 55;
   error (EXIT_FAILURE, 0, _("invalid target_version argument to compile_java_class"));
   return 0;
 }
 
+/* Return the source version to pass to javac.  */
+static const char *
+get_source_version_for_javac (const char *source_version,
+                              const char *target_version)
+{
+  /* The javac option '-source 1.5' has the same meaning as '-source 1.6',
+     but since Java 9 supports only the latter, prefer the latter if a
+     target_version >= 1.6 is requested.  */
+  if (strcmp (source_version, "1.5") == 0
+      && !(target_version[0] == '1' && target_version[1] == '.'
+           && (target_version[2] >= '1' && target_version[2] <= '5')
+           && target_version[3] == '\0'))
+    return "1.6";
+  return source_version;
+}
+
 /* ======================== Compilation subroutines ======================== */
 
 /* Try to compile a set of Java sources with $JAVAC.
@@ -207,7 +288,7 @@ compile_using_envjavac (const char *javac,
   bool err;
   unsigned int command_length;
   char *command;
-  char *argv[4];
+  const char *argv[4];
   int exitstatus;
   unsigned int i;
   char *p;
@@ -258,12 +339,13 @@ compile_using_envjavac (const char *javac,
   if (verbose)
     printf ("%s\n", command);
 
-  argv[0] = "/bin/sh";
+  argv[0] = BOURNE_SHELL;
   argv[1] = "-c";
   argv[2] = command;
   argv[3] = NULL;
-  exitstatus = execute (javac, "/bin/sh", argv, false, false, false,
-                        null_stderr, true, true, NULL);
+  exitstatus = execute (javac, BOURNE_SHELL, argv, NULL,
+                        false, false, false, null_stderr,
+                        true, true, NULL);
   err = (exitstatus != 0);
 
   freea (command);
@@ -285,8 +367,8 @@ compile_using_gcj (const char * const *java_sources,
 {
   bool err;
   unsigned int argc;
-  char **argv;
-  char **argp;
+  const char **argv;
+  const char **argp;
   char *fsource_arg;
   char *ftarget_arg;
   int exitstatus;
@@ -296,7 +378,7 @@ compile_using_gcj (const char * const *java_sources,
     2 + (no_assert_option ? 1 : 0) + (fsource_option ? 1 : 0)
     + (ftarget_option ? 1 : 0) + (optimize ? 1 : 0) + (debug ? 1 : 0)
     + (directory != NULL ? 2 : 0) + java_sources_count;
-  argv = (char **) xmalloca ((argc + 1) * sizeof (char *));
+  argv = (const char **) xmalloca ((argc + 1) * sizeof (const char *));
 
   argp = argv;
   *argp++ = "gcj";
@@ -328,10 +410,10 @@ compile_using_gcj (const char * const *java_sources,
   if (directory != NULL)
     {
       *argp++ = "-d";
-      *argp++ = (char *) directory;
+      *argp++ = directory;
     }
   for (i = 0; i < java_sources_count; i++)
-    *argp++ = (char *) java_sources[i];
+    *argp++ = java_sources[i];
   *argp = NULL;
   /* Ensure argv length was correctly calculated.  */
   if (argp - argv != argc)
@@ -344,7 +426,8 @@ compile_using_gcj (const char * const *java_sources,
       free (command);
     }
 
-  exitstatus = execute ("gcj", "gcj", argv, false, false, false, null_stderr,
+  exitstatus = execute ("gcj", "gcj", argv, NULL,
+                        false, false, false, null_stderr,
                         true, true, NULL);
   err = (exitstatus != 0);
 
@@ -370,27 +453,27 @@ compile_using_javac (const char * const *java_sources,
 {
   bool err;
   unsigned int argc;
-  char **argv;
-  char **argp;
+  const char **argv;
+  const char **argp;
   int exitstatus;
   unsigned int i;
 
   argc =
     1 + (source_option ? 2 : 0) + (target_option ? 2 : 0) + (optimize ? 1 : 0)
     + (debug ? 1 : 0) + (directory != NULL ? 2 : 0) + java_sources_count;
-  argv = (char **) xmalloca ((argc + 1) * sizeof (char *));
+  argv = (const char **) xmalloca ((argc + 1) * sizeof (const char *));
 
   argp = argv;
   *argp++ = "javac";
   if (source_option)
     {
       *argp++ = "-source";
-      *argp++ = (char *) source_version;
+      *argp++ = source_version;
     }
   if (target_option)
     {
       *argp++ = "-target";
-      *argp++ = (char *) target_version;
+      *argp++ = target_version;
     }
   if (optimize)
     *argp++ = "-O";
@@ -399,10 +482,10 @@ compile_using_javac (const char * const *java_sources,
   if (directory != NULL)
     {
       *argp++ = "-d";
-      *argp++ = (char *) directory;
+      *argp++ = directory;
     }
   for (i = 0; i < java_sources_count; i++)
-    *argp++ = (char *) java_sources[i];
+    *argp++ = java_sources[i];
   *argp = NULL;
   /* Ensure argv length was correctly calculated.  */
   if (argp - argv != argc)
@@ -415,7 +498,8 @@ compile_using_javac (const char * const *java_sources,
       free (command);
     }
 
-  exitstatus = execute ("javac", "javac", argv, false, false, false,
+  exitstatus = execute ("javac", "javac", argv, NULL,
+                        false, false, false,
                         null_stderr, true, true, NULL);
   err = (exitstatus != 0);
 
@@ -435,15 +519,15 @@ compile_using_jikes (const char * const *java_sources,
 {
   bool err;
   unsigned int argc;
-  char **argv;
-  char **argp;
+  const char **argv;
+  const char **argp;
   int exitstatus;
   unsigned int i;
 
   argc =
     1 + (optimize ? 1 : 0) + (debug ? 1 : 0) + (directory != NULL ? 2 : 0)
     + java_sources_count;
-  argv = (char **) xmalloca ((argc + 1) * sizeof (char *));
+  argv = (const char **) xmalloca ((argc + 1) * sizeof (const char *));
 
   argp = argv;
   *argp++ = "jikes";
@@ -454,10 +538,10 @@ compile_using_jikes (const char * const *java_sources,
   if (directory != NULL)
     {
       *argp++ = "-d";
-      *argp++ = (char *) directory;
+      *argp++ = directory;
     }
   for (i = 0; i < java_sources_count; i++)
-    *argp++ = (char *) java_sources[i];
+    *argp++ = java_sources[i];
   *argp = NULL;
   /* Ensure argv length was correctly calculated.  */
   if (argp - argv != argc)
@@ -470,8 +554,9 @@ compile_using_jikes (const char * const *java_sources,
       free (command);
     }
 
-  exitstatus = execute ("jikes", "jikes", argv, false, false, false,
-                        null_stderr, true, true, NULL);
+  exitstatus = execute ("jikes", "jikes", argv, NULL,
+                        false, false, false, null_stderr,
+                        true, true, NULL);
   err = (exitstatus != 0);
 
   freea (argv);
@@ -492,7 +577,7 @@ write_temp_file (struct temp_dir *tmpdir, const char *file_name,
   FILE *fp;
 
   register_temp_file (tmpdir, file_name);
-  fp = fopen_temp (file_name, "w");
+  fp = fopen_temp (file_name, "we", false);
   if (fp == NULL)
     {
       error (0, errno, _("failed to create \"%s\""), file_name);
@@ -516,7 +601,7 @@ get_classfile_version (const char *compiled_file_name)
   int fd;
 
   /* Open the class file.  */
-  fd = open (compiled_file_name, O_RDONLY | O_BINARY, 0);
+  fd = open (compiled_file_name, O_RDONLY | O_BINARY | O_CLOEXEC, 0);
   if (fd >= 0)
     {
       /* Read its first 8 bytes.  */
@@ -525,7 +610,10 @@ get_classfile_version (const char *compiled_file_name)
           /* Verify the class file signature.  */
           if (header[0] == 0xCA && header[1] == 0xFE
               && header[2] == 0xBA && header[3] == 0xBE)
-            return header[7];
+            {
+              close (fd);
+              return header[7];
+            }
         }
       close (fd);
     }
@@ -547,7 +635,7 @@ is_envjavac_gcj (const char *javac)
          "$JAVAC --version 2>/dev/null | sed -e 1q | grep gcj > /dev/null"  */
       unsigned int command_length;
       char *command;
-      char *argv[4];
+      const char *argv[4];
       pid_t child;
       int fd[1];
       FILE *fp;
@@ -572,12 +660,12 @@ is_envjavac_gcj (const char *javac)
         abort ();
 
       /* Call $JAVAC --version 2>/dev/null.  */
-      argv[0] = "/bin/sh";
+      argv[0] = BOURNE_SHELL;
       argv[1] = "-c";
       argv[2] = command;
       argv[3] = NULL;
-      child = create_pipe_in (javac, "/bin/sh", argv, DEV_NULL, true, true,
-                              false, fd);
+      child = create_pipe_in (javac, BOURNE_SHELL, argv, NULL,
+                              DEV_NULL, true, true, false, fd);
       if (child == -1)
         goto failed;
 
@@ -629,7 +717,7 @@ is_envjavac_gcj43 (const char *javac)
           | sed -e '/^4\.[012]/d' | grep '^[4-9]' >/dev/null"  */
       unsigned int command_length;
       char *command;
-      char *argv[4];
+      const char *argv[4];
       pid_t child;
       int fd[1];
       FILE *fp;
@@ -654,12 +742,12 @@ is_envjavac_gcj43 (const char *javac)
         abort ();
 
       /* Call $JAVAC --version 2>/dev/null.  */
-      argv[0] = "/bin/sh";
+      argv[0] = BOURNE_SHELL;
       argv[1] = "-c";
       argv[2] = command;
       argv[3] = NULL;
-      child = create_pipe_in (javac, "/bin/sh", argv, DEV_NULL, true, true,
-                              false, fd);
+      child = create_pipe_in (javac, BOURNE_SHELL, argv, NULL,
+                              DEV_NULL, true, true, false, fd);
       if (child == -1)
         goto failed;
 
@@ -1055,6 +1143,7 @@ is_envjavac_oldgcj_14_13_usable (const char *javac,
 static bool
 is_envjavac_nongcj_usable (const char *javac,
                            const char *source_version,
+                           const char *source_version_for_javac,
                            const char *target_version,
                            bool *usablep,
                            bool *source_option_p, bool *target_option_p)
@@ -1110,7 +1199,7 @@ is_envjavac_nongcj_usable (const char *javac,
           /* $JAVAC compiled conftest.java successfully.  */
           /* Try adding -source option if it is useful.  */
           char *javac_source =
-            xasprintf ("%s -source %s", javac, source_version);
+            xasprintf ("%s -source %s", javac, source_version_for_javac);
 
           unlink (compiled_file_name);
 
@@ -1162,8 +1251,8 @@ is_envjavac_nongcj_usable (const char *javac,
                                                   tmpdir->dir_name,
                                                   false, false, false, true))
                         /* $JAVAC compiled conftestfail.java successfully, and
-                           "$JAVAC -source $source_version" rejects it.  So the
-                           -source option is useful.  */
+                           "$JAVAC -source $source_version_for_javac" rejects it.
+                           So the -source option is useful.  */
                         resultp->source_option = true;
                     }
                 }
@@ -1194,7 +1283,7 @@ is_envjavac_nongcj_usable (const char *javac,
                  successfully.  */
               /* Try adding -source option if it is useful.  */
               char *javac_target_source =
-                xasprintf ("%s -source %s", javac_target, source_version);
+                xasprintf ("%s -source %s", javac_target, source_version_for_javac);
 
               unlink (compiled_file_name);
 
@@ -1250,7 +1339,7 @@ is_envjavac_nongcj_usable (const char *javac,
                                                       true))
                             /* "$JAVAC -target $target_version" compiled
                                conftestfail.java successfully, and
-                               "$JAVAC -target $target_version -source $source_version"
+                               "$JAVAC -target $target_version -source $source_version_for_javac"
                                rejects it.  So the -source option is useful.  */
                             resultp->source_option = true;
                         }
@@ -1268,7 +1357,7 @@ is_envjavac_nongcj_usable (const char *javac,
                  -target and -source options. (Supported by Sun javac 1.4 and
                  higher.)  */
               char *javac_target_source =
-                xasprintf ("%s -source %s", javac_target, source_version);
+                xasprintf ("%s -source %s", javac_target, source_version_for_javac);
 
               unlink (compiled_file_name);
 
@@ -1280,7 +1369,7 @@ is_envjavac_nongcj_usable (const char *javac,
                   && get_classfile_version (compiled_file_name)
                      <= corresponding_classfile_version (target_version))
                 {
-                  /* "$JAVAC -target $target_version -source $source_version"
+                  /* "$JAVAC -target $target_version -source $source_version_for_javac"
                      compiled conftest.java successfully.  */
                   resultp->source_option = true;
                   resultp->target_option = true;
@@ -1317,7 +1406,7 @@ is_gcj_present (void)
          "gcj --version 2> /dev/null | \
           sed -e 's,^[^0-9]*,,' -e 1q | \
           sed -e '/^3\.[01]/d' | grep '^[3-9]' > /dev/null"  */
-      char *argv[3];
+      const char *argv[3];
       pid_t child;
       int fd[1];
       int exitstatus;
@@ -1325,8 +1414,8 @@ is_gcj_present (void)
       argv[0] = "gcj";
       argv[1] = "--version";
       argv[2] = NULL;
-      child = create_pipe_in ("gcj", "gcj", argv, DEV_NULL, true, true,
-                              false, fd);
+      child = create_pipe_in ("gcj", "gcj", argv, NULL,
+                              DEV_NULL, true, true, false, fd);
       gcj_present = false;
       if (child != -1)
         {
@@ -1433,7 +1522,7 @@ is_gcj_43 (void)
          "gcj --version 2> /dev/null | \
           sed -e 's,^[^0-9]*,,' -e 1q | \
           sed -e '/^4\.[012]/d' | grep '^[4-9]'"  */
-      char *argv[3];
+      const char *argv[3];
       pid_t child;
       int fd[1];
       int exitstatus;
@@ -1441,8 +1530,8 @@ is_gcj_43 (void)
       argv[0] = "gcj";
       argv[1] = "--version";
       argv[2] = NULL;
-      child = create_pipe_in ("gcj", "gcj", argv, DEV_NULL, true, true,
-                              false, fd);
+      child = create_pipe_in ("gcj", "gcj", argv, NULL,
+                              DEV_NULL, true, true, false, fd);
       gcj_43 = false;
       if (child != -1)
         {
@@ -1782,12 +1871,13 @@ is_javac_present (void)
   if (!javac_tested)
     {
       /* Test for presence of javac: "javac 2> /dev/null ; test $? -le 2"  */
-      char *argv[2];
+      const char *argv[2];
       int exitstatus;
 
       argv[0] = "javac";
       argv[1] = NULL;
-      exitstatus = execute ("javac", "javac", argv, false, false, true, true,
+      exitstatus = execute ("javac", "javac", argv, NULL,
+                            false, false, true, true,
                             true, false, NULL);
       javac_present = (exitstatus == 0 || exitstatus == 1 || exitstatus == 2);
       javac_tested = true;
@@ -1800,7 +1890,9 @@ is_javac_present (void)
    -target option.
    Return a failure indicator (true upon error).  */
 static bool
-is_javac_usable (const char *source_version, const char *target_version,
+is_javac_usable (const char *source_version,
+                 const char *source_version_for_javac,
+                 const char *target_version,
                  bool *usablep, bool *source_option_p, bool *target_option_p)
 {
   /* The cache depends on the source_version and target_version.  */
@@ -1845,7 +1937,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
       java_sources[0] = conftest_file_name;
       if (!compile_using_javac (java_sources, 1,
-                                false, source_version,
+                                false, source_version_for_javac,
                                 false, target_version,
                                 tmpdir->dir_name, false, false, false, true)
           && stat (compiled_file_name, &statbuf) >= 0
@@ -1858,7 +1950,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
           java_sources[0] = conftest_file_name;
           if (!compile_using_javac (java_sources, 1,
-                                    true, source_version,
+                                    true, source_version_for_javac,
                                     false, target_version,
                                     tmpdir->dir_name, false, false, false, true)
               && stat (compiled_file_name, &statbuf) >= 0
@@ -1891,7 +1983,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
                   java_sources[0] = conftest_file_name;
                   if (!compile_using_javac (java_sources, 1,
-                                            false, source_version,
+                                            false, source_version_for_javac,
                                             false, target_version,
                                             tmpdir->dir_name,
                                             false, false, false, true)
@@ -1901,13 +1993,13 @@ is_javac_usable (const char *source_version, const char *target_version,
 
                       java_sources[0] = conftest_file_name;
                       if (compile_using_javac (java_sources, 1,
-                                               true, source_version,
+                                               true, source_version_for_javac,
                                                false, target_version,
                                                tmpdir->dir_name,
                                                false, false, false, true))
                         /* javac compiled conftestfail.java successfully, and
-                           "javac -source $source_version" rejects it.  So the
-                           -source option is useful.  */
+                           "javac -source $source_version_for_javac" rejects it.
+                           So the -source option is useful.  */
                         resultp->source_option = true;
                     }
                 }
@@ -1923,7 +2015,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
           java_sources[0] = conftest_file_name;
           if (!compile_using_javac (java_sources, 1,
-                                    false, source_version,
+                                    false, source_version_for_javac,
                                     true, target_version,
                                     tmpdir->dir_name,
                                     false, false, false, true)
@@ -1938,7 +2030,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
               java_sources[0] = conftest_file_name;
               if (!compile_using_javac (java_sources, 1,
-                                        true, source_version,
+                                        true, source_version_for_javac,
                                         true, target_version,
                                         tmpdir->dir_name,
                                         false, false, false, true)
@@ -1973,7 +2065,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
                       java_sources[0] = conftest_file_name;
                       if (!compile_using_javac (java_sources, 1,
-                                                false, source_version,
+                                                false, source_version_for_javac,
                                                 true, target_version,
                                                 tmpdir->dir_name,
                                                 false, false, false, true)
@@ -1983,13 +2075,13 @@ is_javac_usable (const char *source_version, const char *target_version,
 
                           java_sources[0] = conftest_file_name;
                           if (compile_using_javac (java_sources, 1,
-                                                   true, source_version,
+                                                   true, source_version_for_javac,
                                                    true, target_version,
                                                    tmpdir->dir_name,
                                                    false, false, false, true))
                             /* "javac -target $target_version" compiled
                                conftestfail.java successfully, and
-                               "javac -target $target_version -source $source_version"
+                               "javac -target $target_version -source $source_version_for_javac"
                                rejects it.  So the -source option is useful.  */
                             resultp->source_option = true;
                         }
@@ -2008,7 +2100,7 @@ is_javac_usable (const char *source_version, const char *target_version,
 
               java_sources[0] = conftest_file_name;
               if (!compile_using_javac (java_sources, 1,
-                                        true, source_version,
+                                        true, source_version_for_javac,
                                         true, target_version,
                                         tmpdir->dir_name,
                                         false, false, false, true)
@@ -2016,7 +2108,7 @@ is_javac_usable (const char *source_version, const char *target_version,
                   && get_classfile_version (compiled_file_name)
                      <= corresponding_classfile_version (target_version))
                 {
-                  /* "javac -target $target_version -source $source_version"
+                  /* "javac -target $target_version -source $source_version_for_javac"
                      compiled conftest.java successfully.  */
                   resultp->source_option = true;
                   resultp->target_option = true;
@@ -2046,12 +2138,13 @@ is_jikes_present (void)
   if (!jikes_tested)
     {
       /* Test for presence of jikes: "jikes 2> /dev/null ; test $? = 1"  */
-      char *argv[2];
+      const char *argv[2];
       int exitstatus;
 
       argv[0] = "jikes";
       argv[1] = NULL;
-      exitstatus = execute ("jikes", "jikes", argv, false, false, true, true,
+      exitstatus = execute ("jikes", "jikes", argv, NULL,
+                            false, false, true, true,
                             true, false, NULL);
       jikes_present = (exitstatus == 0 || exitstatus == 1);
       jikes_tested = true;
@@ -2087,10 +2180,14 @@ compile_java_class (const char * const *java_sources,
         bool target_option = false;
         bool fsource_option = false;
         bool ftarget_option = false;
+        const char *source_version_for_javac;
 
         if (target_version == NULL)
           target_version = default_target_version ();
 
+        source_version_for_javac =
+          get_source_version_for_javac (source_version, target_version);
+
         if (is_envjavac_gcj (javac))
           {
             /* It's a version of gcj.  */
@@ -2137,7 +2234,9 @@ compile_java_class (const char * const *java_sources,
           {
             /* It's not gcj.  Assume the classfile versions are correct.  */
             if (is_envjavac_nongcj_usable (javac,
-                                           source_version, target_version,
+                                           source_version,
+                                           source_version_for_javac,
+                                           target_version,
                                            &usable,
                                            &source_option, &target_option))
               {
@@ -2161,7 +2260,7 @@ compile_java_class (const char * const *java_sources,
                : xasprintf ("%s%s%s%s%s%s%s%s%s",
                             javac,
                             source_option ? " -source " : "",
-                            source_option ? source_version : "",
+                            source_option ? source_version_for_javac : "",
                             target_option ? " -target " : "",
                             target_option ? target_version : "",
                             fsource_option ? " -fsource=" : "",
@@ -2270,11 +2369,16 @@ compile_java_class (const char * const *java_sources,
       bool usable = false;
       bool source_option = false;
       bool target_option = false;
+      const char *source_version_for_javac;
 
       if (target_version == NULL)
         target_version = default_target_version ();
 
-      if (is_javac_usable (source_version, target_version,
+      source_version_for_javac =
+        get_source_version_for_javac (source_version, target_version);
+
+      if (is_javac_usable (source_version, source_version_for_javac,
+                           target_version,
                            &usable, &source_option, &target_option))
         {
           err = true;
@@ -2294,7 +2398,7 @@ compile_java_class (const char * const *java_sources,
                            verbose);
 
           err = compile_using_javac (java_sources, java_sources_count,
-                                     source_option, source_version,
+                                     source_option, source_version_for_javac,
                                      target_option, target_version,
                                      directory, optimize, debug, verbose,
                                      false);
