@@ -1,5 +1,5 @@
 /* Filtering of data through a subprocess.  -*- coding: utf-8 -*-
-   Copyright (C) 2009-2016 Free Software Foundation, Inc.
+   Copyright (C) 2009-2022 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2009,
    and Paolo Bonzini <bonzini@gnu.org>, 2009.
 
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _PIPE_FILTER_H
 #define _PIPE_FILTER_H
@@ -47,7 +47,7 @@ extern "C" {
 
    The side that writes data to the subprocess can be seen as a "generator",
    that is, as a subroutine that produces and writes a piece of data here and
-   there, see <http://en.wikipedia.org/wiki/Generator_(computer_science)>.
+   there, see <https://en.wikipedia.org/wiki/Generator_(computer_science)>.
    But often, it can be written in the form of an "iterator", that is, as a
    function that, each time it is invoked, produces and writes one more piece
    of data.
@@ -70,8 +70,8 @@ extern "C" {
 
    The last one uses threads in order to implement two generators running at
    the same time.  (For the relation between generators, coroutines, and
-   threads, see <http://en.wikipedia.org/wiki/Generator_(computer_science)>
-   and <http://en.wikipedia.org/wiki/Coroutine>.)  It is therefore only
+   threads, see <https://en.wikipedia.org/wiki/Generator_(computer_science)>
+   and <https://en.wikipedia.org/wiki/Coroutine>.)  It is therefore only
    portable to platforms with kernel-based POSIX threads.  */
 
 /* These two functions together describe the side that writes data to the
@@ -133,7 +133,8 @@ typedef void (*done_read_fn) (void *data_read, size_t num_bytes_read,
    - the positive exit code of the subprocess if that failed.  */
 extern int
        pipe_filter_ii_execute (const char *progname,
-                               const char *prog_path, const char **prog_argv,
+                               const char *prog_path,
+                               const char * const *prog_argv,
                                bool null_stderr, bool exit_on_error,
                                prepare_write_fn prepare_write,
                                done_write_fn done_write,
@@ -151,6 +152,23 @@ struct pipe_filter_ig;
 
 struct pipe_filter_gi;
 
+/* Finish reading the output via the prepare_read/done_read functions
+   specified to pipe_filter_gi_create.
+
+   Note that the prepare_read/done_read functions may be called in a
+   different thread than the current thread (depending on the platform).
+   However, they will always be called before pipe_filter_gi_close has
+   returned.
+
+   The write side of the pipe is closed as soon as pipe_filter_gi_close
+   starts, while the read side will be closed just before it finishes.
+
+   Return 0 upon success, or (only if exit_on_error is false):
+   - -1 with errno set upon failure,
+   - the positive exit code of the subprocess if that failed.  */
+extern int
+       pipe_filter_gi_close (struct pipe_filter_gi *filter);
+
 /* Create a subprocess and pipe some data through it.
    Arguments:
    - progname is the program name used in error messages.
@@ -179,11 +197,13 @@ struct pipe_filter_gi;
    Return the freshly created 'struct pipe_filter_gi'.  */
 extern struct pipe_filter_gi *
        pipe_filter_gi_create (const char *progname,
-                              const char *prog_path, const char **prog_argv,
+                              const char *prog_path,
+                              const char * const *prog_argv,
                               bool null_stderr, bool exit_on_error,
                               prepare_read_fn prepare_read,
                               done_read_fn done_read,
-                              void *private_data);
+                              void *private_data)
+  _GL_ATTRIBUTE_DEALLOC (pipe_filter_gi_close, 1);
 
 /* Write size bytes starting at buf into the pipe and in the meanwhile
    possibly call the prepare_read and done_read functions specified to
@@ -205,23 +225,6 @@ extern int
        pipe_filter_gi_write (struct pipe_filter_gi *filter,
                              const void *buf, size_t size);
 
-/* Finish reading the output via the prepare_read/done_read functions
-   specified to pipe_filter_gi_create.
-
-   Note that the prepare_read/done_read functions may be called in a
-   different thread than the current thread (depending on the platform).
-   However, they will always be called before pipe_filter_gi_close has
-   returned.
-
-   The write side of the pipe is closed as soon as pipe_filter_gi_close
-   starts, while the read side will be closed just before it finishes.
-
-   Return 0 upon success, or (only if exit_on_error is false):
-   - -1 with errno set upon failure,
-   - the positive exit code of the subprocess if that failed.  */
-extern int
-       pipe_filter_gi_close (struct pipe_filter_gi *filter);
-
 
 /* ============================ pipe_filter_gg ============================ */
 
